<?php
namespace App\Controller;

use Psr\Http\Message\ResponseInterface as Response;
use Psr\Http\Message\ServerRequestInterface as Request;

use App\Templates;
use App\Exception;

class MenuController {
    
    public function __construct (\Doctrine\DBAL\Connection $connection,
                                 \App\Service\MenuService $menuService,
                                 \App\Util\Util $util) {

        $this->menuService = $menuService;
        $this->util        = $util;
    }
    
    public function loadMenu(Request $request, Response $response) {
        ini_set('memory_limit', '-1');
        try {
            
			$row = $request->getParsedBody();
			
            if(empty($row)) 
                throw new \Exception("Empty body request.");

			$CDFILIAL = $row["branchId"];
			
			$data     = $this->menuService->loadMenu($CDFILIAL);
        
            return $response->withJson([
                    'success' => true, 
                    'menu' => $data
            ]);

		} catch (\Exception $e) {
			return $response->withStatus(400)->withJson([
                'success'      => false, 
                'errorMessage' => $e->getMessage()
            ]);
		}
    }

    
    public function getBranchs(Request $request, Response $response) {
        try {

			$data = $this->menuService->getBranchs();

            return $response->withJson($data);
		} catch (\Exception $e) {
			return $response->withStatus(400)->withJson([
                'success'      => false, 
                'errorMessage' => $e->getMessage()
            ]);
		}
    }

    public function getMenus(Request $request, Response $response) {
        try {
			$data = $this->menuService->getMenus();
 
            return $response->withJson($data);
		} catch (\Exception $e) {
			return $response->withStatus(400)->withJson([
                'success'      => false, 
                'errorMessage' => $e->getMessage()
            ]);
		}
    }

    public function getProducts(Request $request, Response $response) {
        try {
            $params = $request->getParsedBody();
            
            $cdFilial   = $params['branchId'];
            $nrConfTela = $params['menuId'];
            
			$data = $this->menuService->getProducts($cdFilial, $nrConfTela);
 
            return $response->withJson($data);
		} catch (\Exception $e) {
			return $response->withStatus(400)->withJson([
                'success'      => false, 
                'errorMessage' => $e->getMessage()
            ]);
		}
    }
    
    public function getMenusUpdates(Request $request, Response $response) {
        try {
			$data = $this->menuService->getMenusUpdates();
 
            return $response->withJson($data);
		} catch (\Exception $e) {
			return $response->withStatus(400)->withJson([
                'success'      => false, 
                'errorMessage' => $e->getMessage()
            ]);
		}
    }
    
    public function getPricingTables(Request $request, Response $response) {
        try {
			$data = $this->menuService->getPricingTables();
 
            return $response->withJson($data);
		} catch (\Exception $e) {
			return $response->withStatus(400)->withJson([
                'success'      => false, 
                'errorMessage' => $e->getMessage()
            ]);
		}
    }
    
    public function getProductsByPricingTable(Request $request, Response $response) {
        try {
            $params = $request->getParsedBody();
            
            $cdTabePrec = $params['tableId'];
            
			$data = $this->menuService->getProductsByPricingTable($cdTabePrec);
 
            return $response->withJson($data);
		} catch (\Exception $e) {
			return $response->withStatus(400)->withJson([
                'success'      => false, 
                'errorMessage' => $e->getMessage()
            ]);
		}
    }
    
    
}